
//	File: GenericServer.java
//	Author: Thomas Lea  (leat@goodnet.com,  http://www.goodnet.com/~leat)
//	Date: 12/7/95
//	Version: 1.0  (for Beta JDK API)
//
// Copyright (c) 1995 Thomas Lea. All Rights Reserved.
//


//	Please feel free to take this program and modify it to your hearts content.
//	The only thing I ask is that my name and the above version/copyright information
//	stay at the top of this module.
//	If you like this example, please send me some mail and let me know
//	how it helped!

import java.net.*;
import java.io.*;
import java.util.Vector;

class GenericServer
{
	private static final int DEFAULT_PORT=5678;
	private ConnectionManager cm = null;

	public GenericServer(int port)
	{
		System.out.println("Server is initializing to port " + port);

		cm = new ConnectionManager(port);
		cm.start();
	}

	public static void main(String args[])
	{
		int server_port;

		try
		{
			server_port = Integer.parseInt(args[0],10);
		}
		catch(Exception e)
		{
			System.out.println("Defaulting to port " + DEFAULT_PORT);
			server_port = DEFAULT_PORT;
		}
		new GenericServer(server_port);
	}
}

// This guy waits for a connection then spawns a ServerConnection to deal with it
class ConnectionManager extends Thread
{
	private static int _port;
	private static Vector _my_threads = new Vector(5,2); //size of 5 initially, grow by 2
	private ServerSocket _main_socket = null;

	public ConnectionManager(int port)
	{
		_port 				  = port;
	}

	public void run()
	{
		serveRequests();
	}

	private void serveRequests()
	{

		try {_main_socket = new ServerSocket(_port);}
		catch(Exception e) { System.err.println(e); System.exit(1);}

		ServerConnection temp_sc = null;

		while (true)
		{
			try
			{
				Socket this_connection = _main_socket.accept();

				
				if (this_connection.getInetAddress().getHostName().equals(this_connection.getLocalAddress().getHostName())){
				    //local connection
				    System.out.println("Local");
                    temp_sc = new TangoConnection(this_connection,this);
				    System.out.println("Created");
				    temp_sc.start();
				    System.out.println("Started");
				    _my_threads.addElement(temp_sc);
				    tango=temp_sc;
				}
				else {//palm
                    temp_sc = new PalmConnection(this_connection,this);
				    temp_sc.start();
				    _my_threads.addElement(temp_sc);
				    palm=temp_sc;
				}

				//clean up the vector if needed
				for(int i=0;i<ConnectionManager._my_threads.size();i++)
					if(!((ServerConnection)(_my_threads.elementAt(i))).isAlive())
						_my_threads.removeElementAt(i);
			}
			catch(Exception e)
			{
				System.err.println("Exception1:\n" );
				e.printStackTrace();
			}
		}
	}
	
	protected void signal() {
	    if (palmConnected())
	        while (content.size()>0) {
	            byte[] b=(byte[])content.elementAt(0);
	            content.removeElementAt(0);
	            System.out.println("Trying to send "+new String(b));
	            palm.send((byte)b.length,b);
	        }
	}
	
	private boolean palmConnected() {
	    if (palm!=null && palm.isAlive()) {
	        System.out.println("palm connected");
	        return true;
	    }
	    else {
	        System.out.println("palm not connected");
	        return false;
	    }
	}

	public synchronized void distributeToPalm(ServerConnection sc,byte in,byte[] b) {
        ServerConnection scon;
//        System.out.println("distributing "+new String(b,0));
        content.addElement(b);
        signal();
        /*
	    for(int i=0;i<ConnectionManager._my_threads.size();i++) {
	        scon=(ServerConnection)(_my_threads.elementAt(i));
		    if(scon.isAlive() &&  scon!=sc && scon.getUserName().equals(sc.getUserName()))
						scon.send(in,b);
		}*/
	}

	public synchronized void distributeToTango(ServerConnection sc,byte in,byte[] b) {
        tango.send(in, b);
	}

    private Vector content=new Vector(10,5);
    private ServerConnection palm, tango;
}

class ServerConnection extends Thread
{
	protected Socket			_mysocket;
	protected DataOutputStream 	_output;
	protected DataInputStream		_input;
	protected ConnectionManager _owner;
	protected String name;

	public ServerConnection(Socket s,ConnectionManager cm)
	{
		_mysocket = s;
		_owner=cm;
	}
	
	public String getUserName() {
	    return name;
	}

	protected void doServerWork()
	{
		//This is where the server actually does its work.
		//when this method finishes, the socket will be closed
		//and this thread will exit.

		//This is just some junk... put your real work here.
      try
		{
            byte n=_input.readByte();
            byte[] bn=new byte[n];
            _input.readFully(bn);
            name=new String(bn);
            while(true) {
                byte i=_input.readByte();
                System.out.println("GOT BYTE:"+i);
                byte[] b=new byte[i];
                _input.readFully(b);
                String t=new String(b);
                System.out.println("Server got "+t);
//                _owner.distribute(this,i,b);
            }
		}
		catch(Exception e)
		{
		    e.printStackTrace();
		}
	}

	public void send(byte i,byte[] b) {
		try
		{
            _output.writeByte(i);
            _output.write(b);
		}
		catch ( Exception e )
		{
			System.err.println( "Exception2:\n" + e );
		}
	}

	public void run()
	{
		System.out.println("Connected to: " + _mysocket.getInetAddress() +":"+ _mysocket.getPort());
		try
		{
			_output = new DataOutputStream(_mysocket.getOutputStream());
			_input = new DataInputStream(_mysocket.getInputStream());

			//Lets get busy!
			doServerWork();

			//We are outta here....
			_mysocket.close();
		}
		catch ( Exception e )
		{
			System.err.println( "Exception3:\n" + e );
		}

		System.out.println("Disconnecting: " + _mysocket.getInetAddress() +":"+ _mysocket.getPort());
		stop();
	}
}

class TangoConnection extends ServerConnection
{
	public TangoConnection(Socket s,ConnectionManager cm)	{
		super(s,cm);
        System.out.println("Tango thread constructor");

	}
	
	protected void doServerWork()
	{
		//This is where the server actually does its work.
		//when this method finishes, the socket will be closed
		//and this thread will exit.

		    System.out.println("Tango thread starting");
      try
		{
            byte n=_input.readByte();
            byte[] bn=new byte[n];
            _input.readFully(bn);
            name=new String(bn);
		    System.out.println("Tango thread got name "+name);
            while(true) {
                byte i=_input.readByte();
                System.out.println("GOT BYTE:"+i);
                byte[] b=new byte[i];
                _input.readFully(b);
                String t=new String(b);
                System.out.println("Server got "+t);
                _owner.distributeToPalm(this,i,b);
            }
		}
		catch(Exception e)
		{
		    e.printStackTrace();
		}
	}

}

class PalmConnection extends ServerConnection
{
	public PalmConnection(Socket s,ConnectionManager cm)	{
		super(s,cm);
	}
	
	protected void doServerWork()
	{
		//This is where the server actually does its work.
		//when this method finishes, the socket will be closed
		//and this thread will exit.

      try
		{
		    System.out.println("Palm thread starting");
            byte n=_input.readByte();
            byte[] bn=new byte[n];
            _input.readFully(bn);
            name=new String(bn);
		    System.out.println("Palm thread got name "+name);
		    _owner.signal();
            while(true) {
                byte i=_input.readByte();
                System.out.println("GOT BYTE:"+i);
                byte[] b=new byte[i];
                _input.readFully(b);
                String t=new String(b);
                System.out.println("Server got "+t);
                _owner.distributeToTango(this,i,b);
            }
		}
		catch(Exception e)
		{
		    e.printStackTrace();
		}
	}

}